<?php

namespace Varioous\Gridexample\Form\Element;

/**
 * Copyright notice
 *
 * (c) 2019 Georg Wurz <georg.wurz@gmail.com>
 *
 * All rights reserved
 *
 * This script is part of the TYPO3 project. The TYPO3 project is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * The GNU General Public License can be found at
 * https://www.gnu.org/licenses/gpl-3.0.en.html.
 *
 * This script is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * This copyright notice MUST APPEAR in all copies of the script!
 */

use TYPO3\CMS\Backend\Form\Element\AbstractFormElement;
use TYPO3\CMS\Backend\Utility\BackendUtility;
use TYPO3\CMS\Core\TypoScript\Parser\TypoScriptParser;
use TYPO3\CMS\Core\Utility\GeneralUtility;

/**
 * Class GridWizardElement
 *
 * @package Varioous\Gridexample\Backend\View\Wizard\Element
 */
class GridWizardElement extends AbstractFormElement
{
    /**
     * @var array
     */
    protected $areas = [];

    /**
     * @var int
     */
    protected $colCount = 12;

    /**
     * @var int
     */
    protected $areaCount = 1;

    /**
     * @var int
     */
    protected $rowCount = 1;

    /**
     * @var string
     */
    protected $fieldName = '';

    /**
     * @return array
     */
    public function render()
    {
        // check if the grid is inside another grid and the current grid is not a "gridelement"
        if($this->data['databaseRow']['tx_gridelements_container'][0] && !$this->data['databaseRow']['tx_gridelements_backend_layout']){
            return;
        }

        $resultArray = $this->initializeResultArray();
        $this->init();

        $fieldInformationResult = $this->renderFieldInformation();
        $fieldInformationHtml = $fieldInformationResult['html'];
        $resultArray = $this->mergeChildReturnIntoExistingResult($resultArray, $fieldInformationResult, false);

        $fieldWizardResult = $this->renderFieldWizard();
        $fieldWizardHtml = $fieldWizardResult['html'];
        $resultArray = $this->mergeChildReturnIntoExistingResult($resultArray, $fieldWizardResult, false);

        $json = json_encode($this->areas, JSON_HEX_QUOT | JSON_HEX_TAG | JSON_HEX_AMP | JSON_HEX_APOS);
        $html = [];
        $html[] =   '<div class="formengine-field-item t3js-formengine-field-item">';
        $html[] =       $fieldInformationHtml;
        $html[] =       '<div class="form-control-wrap">';
        $html[] =           '<div class="form-wizards-wrap">';
        $html[] =               '<div class="form-wizards-element">';
        $html[] =                   '<input';
        $html[] =                       ' type="hidden"';
        $html[] =                       ' name="' . htmlspecialchars($this->data['parameterArray']['itemFormElName']) . '"';
        $html[] =                       ' value="' . htmlspecialchars($this->data['parameterArray']['itemFormElValue']) . '"';
        $html[] =                   '/>';
        $html[] =                   '<style>#grideditor-' . $this->fieldName . ' *,
                                        #grideditor-' . $this->fieldName . ' *::after,
                                        #grideditor-' . $this->fieldName . ' *::before {
                                          box-sizing: inherit;
                                        }
                                    
                                        #grideditor-' . $this->fieldName . ' {
                                          box-sizing: border-box;
                                        }
                                    </style>';
        $html[] =                   '<div id="grideditor-' . $this->fieldName . '"';
        $html[] =                       ' data-field="' . htmlspecialchars($this->data['parameterArray']['itemFormElName']) . '"';
        $html[] =                       ' data-areas="' . htmlspecialchars($json) . '"';
        $html[] =                       ' data-rowcount="' . (int)$this->rowCount . '"';
        $html[] =                       ' data-colcount="' . (int)$this->colCount . '"';
        $html[] =                       ' data-areacount="' . (int)$this->areaCount . '">';
        $html[] =                   '</div>';
        $html[] =               '</div>';
        $html[] =               '<div class="form-wizards-items-bottom">';
        $html[] =                   $fieldWizardHtml;
        $html[] =               '</div>';
        $html[] =           '</div>';
        $html[] =       '</div>';
        $html[] =   '</div>';

        $html = implode(LF, $html);
        $resultArray['html'] = $html;
        $resultArray['requireJsModules'][] = ['TYPO3/CMS/Gridexample/grideditor/GridEditor' => 'function(GridEditor) { new GridEditor("' . htmlspecialchars($this->fieldName) . '") } '];

        return $resultArray;
    }

    /**
     * Initialize wizard
     *
     * Set default values via TSConfig, e.g.
     *
     * TCAdefaults.tt_content.pi_flexform.data.sDEF.grid.myCustomContentElement (
     * c1 {
     *      column {
     *          start = 4
     *          end = 10
     *          span = 6
     *      }
     *      row {
     *          start = 1
     *          end = 2
     *          span = 1
     *      }
     * }
     * )
     */
    protected function init()
    {
        $areasDoNotMatchAreaCount = false;
        $areasArray = [];

        if (!empty($this->data['parameterArray']['fieldConf']['config']['parameters']['cols'])) {
            $this->colCount = (int)$this->data['parameterArray']['fieldConf']['config']['parameters']['cols'];
        }

        if (!empty($this->data['parameterArray']['fieldConf']['config']['parameters']['areas'])) {
            $this->areaCount = (int)$this->data['parameterArray']['fieldConf']['config']['parameters']['areas'];
        }

        $isFlexform = is_array($this->data['databaseRow'][$this->data['fieldName']])?true:false;

        if($isFlexform) {
            $dbValue = $this->data['flexFormRowData'][$this->data['flexFormFieldName']]['vDEF'];
            $this->fieldName = $this->data['flexFormFieldName'];
        } else {
            $dbValue = $this->data['databaseRow'][$this->data['fieldName']];
            $this->fieldName = $this->data['fieldName'];
        }

        // TS parser
        $parser = GeneralUtility::makeInstance(TypoScriptParser::class);

        if($dbValue){
            // load TS parser
            $parser->parse($dbValue);
            $data = $parser->setup['grid.'];
            $areas = $data['areas.'];

            if(count($areas) != (int)$this->areaCount) {
                $areasDoNotMatchAreaCount = true;
            }
        } else {
            $pageTsConfig = BackendUtility::getPagesTSconfig($this->data['effectivePid']);

            if($isFlexform) {
                $tsConfigValue = $pageTsConfig['TCAdefaults.'][$this->data['tableName'] . '.'][$this->data['fieldName'] . '.']['data.'][$this->data['flexFormSheetName'] . '.'][$this->data['flexFormFieldName'] . '.'][$this->data['databaseRow']['CType'][0]];
            } else {
                $tsConfigValue = $pageTsConfig['TCAdefaults.'][$this->data['tableName'] . '.'][$this->data['fieldName'] . '.'][$this->data['databaseRow']['CType'][0]];
            }

            if($tsConfigValue) {
                // load TS parser
                $parser->parse($tsConfigValue);
                $areas = $parser->setup;

                if(count($areas) != (int)$this->areaCount) {
                    $areasDoNotMatchAreaCount = true;
                }
            }
        }

        if($areas && !$areasDoNotMatchAreaCount) {
            foreach ($areas as $areaKey => $area) {
                $areasArray[str_replace('.', '', $areaKey)]['column'] = [
                    'start' => (int)$area['column.']['start'],
                    'end' => (int)$area['column.']['end'],
                    'span' => (int)$area['column.']['span'],
                ];

                $areasArray[str_replace('.', '', $areaKey)]['row'] = [
                    'start' => (int)$area['row.']['start'],
                    'end' => (int)$area['row.']['end'],
                    'span' => (int)$area['row.']['span'],
                ];
            }
        } else {
            // no right data from database or areas do not match area count
            for ($i = 0; $i < $this->areaCount; $i++) {
                if ($this->rowCount == $this->areaCount) {
                    // if rows are equal with areas, we put every area in a new row
                    $areasArray['c' . ($i + 1)] = [
                        'column' => [
                            'start' => 1,
                            'end' => (int)($this->colCount + 1),
                            'span' => (int)($this->colCount)
                        ],
                        'row' => [
                            'start' => $i + 1,
                            'end' => $i + 2,
                            'span' => 1
                        ]
                    ];
                } else {
                    // otherwise we divide the columns into one row
                    $areasArray['c' . ($i + 1)] = [
                        'column' => [
                            'start' => (int)(($this->colCount / $this->areaCount) * $i + 1),
                            'end' => (int)($this->colCount / $this->areaCount * ($i + 1) + 1),
                            'span' => (int)($this->colCount / $this->areaCount)
                        ],
                        'row' => [
                            'start' => 1,
                            'end' => 2,
                            'span' => 1
                        ]
                    ];
                }
            }
        }

        $this->areas = $areasArray;
    }
}
