<?php

namespace Varioous\Gridexample\ViewHelpers;

use TYPO3\CMS\Core\TypoScript\Parser\TypoScriptParser;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3Fluid\Fluid\Core\ViewHelper\AbstractViewHelper;

/**
 * Copyright notice
 *
 * (c) 2019 Georg Wurz <georg.wurz@gmail.com>
 *
 * All rights reserved
 *
 * This script is part of the TYPO3 project. The TYPO3 project is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * The GNU General Public License can be found at
 * https://www.gnu.org/licenses/gpl-3.0.en.html.
 *
 * This script is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * This copyright notice MUST APPEAR in all copies of the script!
 */
class TypoScriptParserViewHelper extends AbstractViewHelper
{

    public function initializeArguments()
    {
        parent::initializeArguments();
        $this->registerArgument('typoscript', 'string', 'The typoscript code to be parsed', true);
    }

    /**
     * Renders the TypoScript-Array
     *
     * @return array
     */
    public function render()
    {
        $parser = GeneralUtility::makeInstance(TypoScriptParser::class);
        $parser->parse($this->arguments['typoscript']);

        return $this->removeDots($parser->setup);
    }

    /**
     * Removes dots in the TypoScript-Array
     *
     * @param	array		$settings
     * @return	array
     */
    private function removeDots($settings) {
        $conf = array();
        foreach ($settings as $key => $value) {
            $conf[$this->removeDotAtTheEnd($key)] = is_array($value) ? $this->removeDots($value) : $value;
        }

        return $conf;
    }

    /**
     * Removes a dot at the end of the $string
     *
     * @param	string		$string
     * @return	string		$string
     */
    private function removeDotAtTheEnd($string) {
        return preg_replace('/\.$/', '', $string);
    }
}

?>
